/*
	lda.c
	By Tuxgasy <tuxgasy@tuxfamily.org>
	
	This file is part of Glda.
	
	You may redistribute it and/or modify it under the terms of the
	GNU General Public License, as published by the Free Software
	Foundation; either version 2 of the License, or (at your option) any later version.
	
	Glda is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	See the GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with Glda; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <libxml/tree.h>
#include <libxml/parser.h>
#include <libxml/xpath.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "../inc/main.h"
#include "../inc/magasin.h"
#include "../inc/lda.h"

/* Création d'une liste Lda chainee simple utilisant la librairie GLib */
Lda* initLda(){
	Lda *pLda;
	pLda = NULL;
	return pLda;
}

Lda *resetLda(Lda *pLda){
	if(pLda != NULL){
		freeLdaChainee(pLda);
	}
	pLda = initLda();
	return pLda;
}

/* Ajout d'un article dans la liste chainee a la fin */
Lda* AjoutArticle(Lda *pLda, gint Quantite, gint *TabIdArticle){
	Article *pArticle;
	gint i;
	gint position;
	
	/** On verifie si l'article est deja ajoute ou pas **/
	position = dejaAjouter(pLda, TabIdArticle);
	/* Article deja ajoute. On met a jour la quantite (on augmente d'une) */
	if(position != -1){
		mettreAjourQuantite(pLda, position, 1);
		return pLda;
	}
	/* Article pas encore ajoute. On l'ajoute */
	pArticle = g_malloc(sizeof(Article));
	if(pArticle == NULL){
		fprintf(stderr, "Il n'y a pas d'espace mémoire disponible.\n");
		return pLda;
	}
	pArticle->Quantite = Quantite;
	for(i=0;i<3;i++){
		pArticle->TabIdArticle[i] = TabIdArticle[i];
	}
	/**********************Ajout de Ref***A faire**************/
	pLda = (Lda*)g_slist_append((GSList*)pLda, pArticle);
	return pLda;
}

/* Suppression d'un article dans la liste */
Lda* SupprArticle(Lda *pLda, gint position){
	Article *pArticle;
	
	pArticle = g_slist_nth_data((GSList*)pLda, position);
	if(pArticle == NULL){
		fprintf(stderr, "Article introuvable\n");
		return pLda;
	}
	
	pLda = (Lda*)g_slist_remove((GSList*)pLda, pArticle);
	
	return pLda;
}

/*
	Verification de l'article s'il est deja ajoute
	Retourne -1 si l'article n'est pas encore ajouté
	Sinon retourne la position de l'élément dans la liste.
*/
gint dejaAjouter(Lda *pLda, gint *TabIdArticle){
	Lda *pLdaCourant;
	Article *pArticle;
	gint i;
	
	if(pLda == NULL){
		return -1;
	}
	
	pLdaCourant = (Lda*)g_slist_nth((GSList*)pLda, 0);
	pArticle = g_slist_nth_data((GSList*)pLda, 0);
	i = 1;
	while(/*(pLdaCourant != NULL) ||*/ (memcmp(pArticle->TabIdArticle, TabIdArticle, sizeof(pArticle->TabIdArticle)) != 0)){
		pLdaCourant = (Lda*)g_slist_nth((GSList*)pLda, i);
		pArticle = g_slist_nth_data((GSList*)pLda, i);
		if(pLdaCourant == NULL)
			return -1;
		i++;
	}
	
	if(memcmp(pArticle->TabIdArticle, TabIdArticle, sizeof(pArticle->TabIdArticle)) == 0){
		return i-1;
	}
	
	return -1;
}

/* Mise a jour de la quantite d'un article dans la liste chainee (augmentation/reduction) */
gint mettreAjourQuantite(Lda *pLda, gint position, gint QuantiteAAjouterOuAEnlever){
	Article *pArticle;
	
	pArticle = g_slist_nth_data((GSList*)pLda, position);
	
	pArticle->Quantite = (pArticle->Quantite)+QuantiteAAjouterOuAEnlever;
	
	return pArticle->Quantite;
}

void mettreNouveauQuantite(Lda *pLda, gint position, gint NewQuantite){
	Article *pArticle;
	
	pArticle = g_slist_nth_data((GSList*)pLda, position);
	
	pArticle->Quantite = NewQuantite;
}

/* Obtenir le nombre d'article et le prix total */
void getNbArticlesAndPrixTotal(Lda *pLda, Magasin pMagasin, gint *NbArt, gdouble *PrixTot){
	Lda *pLdaCourant;
	Article *pArticle;
	gchar *pArticlePrice;
	gdouble prixArt;
	gint i;
	
	*NbArt = 0;
	*PrixTot = 0;
	
	pLdaCourant = (Lda*)g_slist_nth((GSList*)pLda, 0);
	pArticle = (Article*)g_slist_nth_data((GSList*)pLda, 0);
	
	i = 1;
	while(pLdaCourant != NULL){
		pArticlePrice = getArticlePriceBySousCategory(pMagasin,
								pArticle->TabIdArticle[0],
								pArticle->TabIdArticle[1],
								pArticle->TabIdArticle[2]);
		
		prixArt = strtod(pArticlePrice, NULL)*(pArticle->Quantite);
		*PrixTot = (*PrixTot)+prixArt;
		*NbArt = (*NbArt) + (pArticle->Quantite);
		
		free(pArticlePrice);
		pLdaCourant = (Lda*)g_slist_nth((GSList*)pLda, i);
		pArticle = g_slist_nth_data((GSList*)pLda, i);
		i++;
	}
	
	/*pArticlePrice = getArticlePriceBySousCategory(pMagasin, pArticle->TabIdArticle[0],
								pArticle->TabIdArticle[1],
								pArticle->TabIdArticle[2]);
	
	prixArt = strtod(pArticlePrice, NULL)*(pArticle->Quantite);
	*PrixTot = (*PrixTot)+prixArt;
	*NbArt = (*NbArt) + (pArticle->Quantite);
	
	free(pArticlePrice);*/
}

/* Ouverture d'une liste sauvegardee */
Lda *openListeLDA(Lda *pLda, xmlDocPtr pXmlFic, gchar *ficLDA, GKeyFile *ConfigLda){
	xmlDocPtr doc;
	xmlNodePtr racine;
	
	gchar *pQuantite;
	gint Quantite;
	
	gint TabIdArticle[3];
	
	gint i;
	
	doc = xmlParseFile(ficLDA);
	if(doc == NULL){
		fprintf(stderr, "Fichier invalide.\n");
		return NULL;
	}
	
	racine = xmlDocGetRootElement(doc);
	if(racine == NULL){
		fprintf(stderr, "Fichier invalide.\n");
		xmlFreeDoc(doc);
		return NULL;
	}
	if(strcmp((char*)racine->name, "lda") != 0){
		fprintf(stderr, "Ce n'est pas un fichier lda.\n");
		xmlFreeDoc(doc);
		return NULL;
	}
	
	/*show(racine);*/
	
	pQuantite = NULL;
	
	i = 1;
	do{
		if(pQuantite != NULL)
			g_free(pQuantite);
		
		
		pQuantite = getQuantiteLDA(doc, i);
		if(pQuantite != NULL){
			getIdArtLDA(doc, i, pXmlFic, TabIdArticle);
			/*printf("%d - %d - %d\n", TabIdArticle[0], TabIdArticle[1], TabIdArticle[2]);*/
			Quantite = strtol(pQuantite, NULL, 10);
			/*printf("Quantite : %d\n", Quantite);*/
			
			pLda = AjoutArticle(pLda, Quantite, TabIdArticle);
		}
		i++;
	}while(pQuantite != NULL);
	
	xmlFreeDoc(doc);
	return pLda;
}

void getIdArtLDA(xmlDocPtr doc, gint id, xmlDocPtr pXmlFic, gint *TabIdArticle){
	xmlXPathContextPtr xmlperms_context = NULL;
	xmlXPathObjectPtr xmlobject;
	const char path_template[] = "/lda/article[%d]";
	xmlChar *path;
	
	gchar *refCat, *refSSCat, *refArt;
	
	xmlXPathInit();
	xmlperms_context = xmlXPathNewContext(doc);
	
	xmlXPathInit();
	xmlperms_context = xmlXPathNewContext(doc);
	
	path = (xmlChar*)g_strdup_printf(path_template, id);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(xmlobject->nodesetval){
		refCat = (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar *) "cat");
		refSSCat = (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar *) "sscat");
		refArt = (gchar*)xmlGetProp(xmlobject->nodesetval->nodeTab[0], (const xmlChar *) "art");
		
		/*printf("%s - %s - %s\n", refCat, refSSCat, refArt);*/
		
		TabIdArticle[0] = getIdCategoryByRef(pXmlFic, refCat);
		TabIdArticle[1] = getIdSSCategoryByRef(pXmlFic, TabIdArticle[0], refSSCat);
		TabIdArticle[2] = getIdArticleByRef(pXmlFic, TabIdArticle[0], TabIdArticle[1], refArt);
		
		/*printf("%d - %d - %d\n", TabIdArticle[0], TabIdArticle[1], TabIdArticle[2]);*/
		
		xmlFree(path);
		xmlXPathFreeObject(xmlobject);
		xmlXPathFreeContext(xmlperms_context);
		g_free(refCat);
		g_free(refSSCat);
		g_free(refArt);
	}
	else{
		xmlFree(path);
		xmlXPathFreeObject(xmlobject);
		xmlXPathFreeContext(xmlperms_context);
	}
}

gchar *getQuantiteLDA(xmlDocPtr doc, gint id){
	xmlXPathContextPtr xmlperms_context = NULL;
	xmlXPathObjectPtr xmlobject;
	const char path_template[] = "/lda/article[%d]/quantite/text()";
	xmlChar *path;
	
	gchar *pQuantite;
	
	xmlXPathInit();
	xmlperms_context = xmlXPathNewContext(doc);
	
	path = (xmlChar*)g_strdup_printf(path_template, id);
	xmlobject = xmlXPathEval(path, xmlperms_context);
	
	if(xmlobject->nodesetval){
		pQuantite = g_strdup_printf("%s", (char *)xmlobject->nodesetval->nodeTab[0]->content);
		
		xmlFree(path);
		xmlXPathFreeObject(xmlobject);
		xmlXPathFreeContext(xmlperms_context);
		return pQuantite;
	}
	else{
		xmlFree(path);
		xmlXPathFreeObject(xmlobject);
		xmlXPathFreeContext(xmlperms_context);
		return NULL;
	}
}

/* Sauvegarde la liste */
gboolean saveListeDefault(Lda *pLda, Magasin pMagasin, gchar *ficLDA, GKeyFile *ConfigLda){
	xmlDocPtr doc;
	xmlNodePtr racine;
	xmlNodePtr artNode, NextArtNode;
	xmlNodePtr quantiteNode, NextQuantiteNode;
	gchar *pQuantite;
	gchar *sText;
	Lda *pLdaCourant;
	Article *pArticle;
	gint i;
	
	doc = xmlNewDoc((const xmlChar*)"1.0");
	racine = xmlNewNode(NULL, (const xmlChar*)"lda");
	artNode = xmlNewNode(NULL, (const xmlChar*)"article");
	quantiteNode = xmlNewNode(NULL, (const xmlChar*)"quantite");
	
	if(pLda == NULL){
		printf("Votre liste d'achat est encore vide\n");
	}
	
	pLdaCourant = (Lda*)g_slist_nth((GSList*)pLda, 0);
	pArticle = g_slist_nth_data((GSList*)pLda, 0);
	
	xmlAddChild((xmlNodePtr)doc, racine);
	
	i = 1;
	if(pLdaCourant != NULL){
		xmlAddChild(racine, artNode);
		
		sText = getRefCategoryById(pMagasin, pArticle->TabIdArticle[0]);
		xmlSetProp(artNode, (const xmlChar*)"cat", (const xmlChar*)sText);
		g_free(sText);
		
		sText = getRefSSCategoryById(pMagasin, pArticle->TabIdArticle[0], pArticle->TabIdArticle[1]);
		xmlSetProp(artNode, (const xmlChar*)"sscat", (const xmlChar*)sText);
		g_free(sText);
		
		sText = getRefArt(pMagasin, pArticle->TabIdArticle[0], pArticle->TabIdArticle[1], pArticle->TabIdArticle[2]);
		xmlSetProp(artNode, (const xmlChar*)"art", (const xmlChar*)sText);
		g_free(sText);
		
		xmlAddChild(artNode, quantiteNode);
		pQuantite = g_strdup_printf("%d", pArticle->Quantite);
		xmlNodeSetContent(quantiteNode, (xmlChar*)pQuantite);
		g_free(pQuantite);
		
		pLdaCourant = (Lda*)g_slist_nth((GSList*)pLda, i);
		pArticle = g_slist_nth_data((GSList*)pLda, i);
		i++;
		
		while(pLdaCourant != NULL){
			NextArtNode = xmlNewNode(NULL, (const xmlChar*)"article");
			NextQuantiteNode = xmlNewNode(NULL, (const xmlChar*)"quantite");
			
			xmlAddSibling(artNode, NextArtNode);
			
			sText = getRefCategoryById(pMagasin, pArticle->TabIdArticle[0]);
			xmlSetProp(NextArtNode, (const xmlChar*)"cat", (const xmlChar*)sText);
			g_free(sText);
			
			sText = getRefSSCategoryById(pMagasin, pArticle->TabIdArticle[0], pArticle->TabIdArticle[1]);
			xmlSetProp(NextArtNode, (const xmlChar*)"sscat", (const xmlChar*)sText);
			g_free(sText);
			
			sText = getRefArt(pMagasin, pArticle->TabIdArticle[0], pArticle->TabIdArticle[1], pArticle->TabIdArticle[2]);
			xmlSetProp(NextArtNode, (const xmlChar*)"art", (const xmlChar*)sText);
			g_free(sText);
			
			xmlAddChild(NextArtNode, NextQuantiteNode);
			pQuantite = g_strdup_printf("%d", pArticle->Quantite);
			xmlNodeSetContent(NextQuantiteNode, (xmlChar*)pQuantite);
			g_free(pQuantite);
			
			artNode = NextArtNode;
			quantiteNode = NextQuantiteNode;
			
			pLdaCourant = (Lda*)g_slist_nth((GSList*)pLda, i);
			pArticle = g_slist_nth_data((GSList*)pLda, i);
			i++;
		}
	}
	
	if(xmlSaveFormatFile(ficLDA, doc, 1) == -1){
		xmlFreeDoc(doc);
		return FALSE;
	}
	xmlFreeDoc(doc);
	return TRUE;
}

/* Sauvegarde la liste sous forme texte simple */
gboolean saveListToTxt(Lda *pLda, Magasin pMagasin, gchar *ficTXT, GKeyFile *ConfigLda){
	Lda *pLdaCourant;
	Article *pArticle;
	gdouble prixArt, prixTotal;
	gint nbArt;
	gchar *pArticleName, *pArticlePrice;
	gint i;
	
	prixTotal = 0;
	nbArt = 0;
	FILE* pFicTXT = NULL;
	gchar *devise;
	
	devise = getUserDevise(ConfigLda);
	
	pFicTXT = fopen(ficTXT, "w");
	
	if(pFicTXT == NULL){
		fprintf(stderr, "Impossible d'ouvrir ou de créer le fichier:\n%s\n", ficTXT);
		g_free(devise);
		return FALSE;
	}
	
	if(pLda == NULL){
		printf("Votre liste d'achat est encore vide\n");
	}
	
	pLdaCourant = (Lda*)g_slist_nth((GSList*)pLda, 0);
	pArticle = g_slist_nth_data((GSList*)pLda, 0);
	i = 1;
	while(pLdaCourant != NULL){
		pArticleName = getArticleNameBySousCategory(pMagasin, 	pArticle->TabIdArticle[0],
									pArticle->TabIdArticle[1],
									pArticle->TabIdArticle[2]);
		pArticlePrice = getArticlePriceBySousCategory(pMagasin,	pArticle->TabIdArticle[0],
									pArticle->TabIdArticle[1],
									pArticle->TabIdArticle[2]);
		
		prixArt = strtod(pArticlePrice, NULL)*(pArticle->Quantite);
		prixTotal = prixTotal+prixArt;
		nbArt = nbArt + (pArticle->Quantite);
		
		fprintf(pFicTXT, "%d : %s \t\t%.2f %s\t%d\n", i, pArticleName, prixArt, devise, pArticle->Quantite);
		
		free(pArticleName);
		free(pArticlePrice);
		
		pLdaCourant = (Lda*)g_slist_nth((GSList*)pLda, i);
		pArticle = g_slist_nth_data((GSList*)pLda, i);
		i++;
	}

	/*pArticleName = getArticleNameBySousCategory(pMagasin, 	pArticle->TabIdArticle[0],
								pArticle->TabIdArticle[1],
								pArticle->TabIdArticle[2]);
	pArticlePrice = getArticlePriceBySousCategory(pMagasin,	pArticle->TabIdArticle[0],
								pArticle->TabIdArticle[1],
								pArticle->TabIdArticle[2]);
	
	prixArt = strtod(pArticlePrice, NULL)*(pArticle->Quantite);
	prixTotal = prixTotal+prixArt;
	nbArt = nbArt + (pArticle->Quantite);
	
	fprintf(pFicTXT, "%d : %s \t\t%.2f %s\t%d\n", i, pArticleName, prixArt, devise, pArticle->Quantite);
	
	free(pArticleName);
	free(pArticlePrice);*/

	fprintf(pFicTXT, "\nNombre d'article(s): %d.\n", nbArt);
	fprintf(pFicTXT, "Prix total: %.2f %s.\n\n", prixTotal, devise);
	
	fclose(pFicTXT);
	g_free(devise);
	return TRUE;
}

/*gboolean saveListeToHtml(listeArticles *teteLdaChainee, Magasin ptrFic, char *title, GKeyFile *ConfigLda){

	return TRUE;
}*/

void freeLdaChainee(Lda *pLda){
	g_slist_free((GSList*)pLda);
}
